@@ -1,7 +1,7 @@
 /*
  * Font scaler
  *
- * $Id: raster_scale.cc,v 1.3 2003/06/25 04:23:54 guillem Exp $
+ * $Id: raster_scale.cc 127 2005-11-03 01:53:01Z guillem $
  *
  * Copyright (C) 1997-1998 Herbert Duerr
  *
@@ -16,8 +16,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Softaware
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -81,7 +81,7 @@ Rasterizer::useTTFont(TTFont* _ttFont, int _flags)
 		if (sizePoints[1])
 			delete[] p[1];
 		sizePoints[1] = MEMSLACK * i;
-		p[1] = new point[sizePoints[1]];
+		p[1] = new Point[sizePoints[1]];
 	}
 
 	i = ttFont->maxpTable->maxContours + ttFont->maxpTable->maxCompContours;
@@ -222,14 +222,14 @@ Rasterizer::getFontExtent(FontExtent *fe)
 		dX = (((fe->xBlackboxMax - 1) | ((8 << ~format) - 1)) + 1) >> 3;
 	}
 
-	U32 buflen = fe->buflen;
-	U8 *endbmp = fe->buffer + buflen;
+	u32_t buflen = fe->buflen;
+	u8_t *endbmp = fe->buffer + buflen;
 
 	CharInfo *ci = (CharInfo *)fe->buffer;
 	fe->bmpFormat = MSB_BIT_FIRST | MSB_BYTE_FIRST;
 	fe->numGlyphs = ttFont->maxpTable->numGlyphs;
-	fe->bitmaps = (U8 *)&ci[fe->numGlyphs];
-	U8 *buffer = fe->bitmaps;
+	fe->bitmaps = (u8_t *)&ci[fe->numGlyphs];
+	u8_t *buffer = fe->bitmaps;
 	if (buffer >= endbmp) {
 		// XXX: not even enough room for charinfo!!! what to do???
 		return;
@@ -263,7 +263,7 @@ Rasterizer::getFontExtent(FontExtent *fe)
 
 
 int
-Rasterizer::putChar8Bitmap(char c, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
+Rasterizer::putChar8Bitmap(char c, u8_t *bmp, u8_t *endbmp, GlyphMetrics *gm)
 {
 	debug("charNo8 = %d", c);
 	int glyphNo = ttFont->getGlyphNo8(c);
@@ -272,7 +272,7 @@ Rasterizer::putChar8Bitmap(char c, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
 
 
 int
-Rasterizer::putChar16Bitmap(int c, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
+Rasterizer::putChar16Bitmap(int c, u8_t *bmp, u8_t *endbmp, GlyphMetrics *gm)
 {
 	int glyphNo = ttFont->getGlyphNo16(c);
 	debug("charNo16 = %d", c);
@@ -281,7 +281,7 @@ Rasterizer::putChar16Bitmap(int c, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
 
 
 int
-Rasterizer::putGlyphBitmap(int glyphNo, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
+Rasterizer::putGlyphBitmap(int glyphNo, u8_t *bmp, u8_t *endbmp, GlyphMetrics *gm)
 {
 	debug("\n=============== glyphNo %d ==================\n", glyphNo);
 
@@ -298,7 +298,7 @@ Rasterizer::putGlyphBitmap(int glyphNo, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
 	int xmin = INT_MAX, ymin = xmin;
 	int xmax = INT_MIN, ymax = xmax;
 
-	register point *pp = p[1];
+	register Point *pp = p[1];
 	for (int i = nPoints[1]; --i >= 0; ++pp) {
 		if (MAGNIFY > 1) {
 			pp->xnow *= MAGNIFY;
@@ -385,7 +385,7 @@ Rasterizer::putGlyphBitmap(int glyphNo, U8 *bmp, U8 *endbmp, GlyphMetrics *gm)
 }
 
 void
-Rasterizer::putGlyphData(int ne, int np, int *ep, point *pp, int glyphNo,
+Rasterizer::putGlyphData(int ne, int np, int *ep, Point *pp, int glyphNo,
 		         int xmin)
 {
 	nEndPoints = ne;
@@ -424,15 +424,12 @@ void
 Rasterizer::scaleGlyph()
 {
 	// scale outline
-	register point *pp = p[1];
+	register Point *pp = p[1];
 	for (int i = nPoints[1]; --i >= 0; ++pp) {
 		pp->xold = scaleX(pp->xnow, pp->ynow);
 		pp->yold = scaleY(pp->ynow, pp->xnow);
 		pp->xnow = pp->xold;
 		pp->ynow = pp->yold;
-#ifdef WIN32
-		pp->xgoal = pp->ygoal = 0;
-#endif
 	}
 
 	switch (status) {
@@ -454,7 +451,7 @@ void
 Rasterizer::printOutline(void)
 {
 	debug("\n=== grid fitted outline ===\n");
-	point *pp = p[1];
+	Point *pp = p[1];
 	for (int i = 0, j = 0; i < nPoints[1] + 2; ++i, ++pp) {
 		debug("p[%d]\t%6d %6d  ", i, pp->xold, pp->yold);
 		debug("-> %6d %6d", pp->xnow, pp->ynow);
@@ -463,15 +460,6 @@ Rasterizer::printOutline(void)
 
 		debug(" %c", (pp->flags & ON_CURVE) ? '*' : ' ');
 
-#ifdef WIN32
-		debug("  (%6d %6d)", pp->xgoal, pp->ygoal);
-
-		debug("  %c%c", (pp->xnow == pp->xgoal) ? '+' : '@',
-			 (pp->ynow == pp->ygoal) ? '+' : '@');
-
-		debug("  %d %d", pp->xnow - pp->xgoal,
-			 pp->ynow - pp->ygoal);
-#endif
 		debug("\n");
 		if (i == endPoints[j]) {
 			++j;
