@@ -1,7 +1,7 @@
 /*
  * Character Location Table
  *
- * $Id: table_loca.cc,v 1.1.1.1 2002/11/14 12:08:11 guillem Exp $
+ * $Id: table_loca.cc 127 2005-11-03 01:53:01Z guillem $
  *
  * Copyright (C) 1997-1998 Herbert Duerr
  *
@@ -16,8 +16,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Softaware
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  */
 
@@ -41,7 +41,7 @@ LocaTable::getGlyphOffset(int glyphNo)
 			ofs = -1;
 	} else {
 		seekAbsolute(glyphNo << 1);
-		U16 ofs16 = readUShort();
+		u16_t ofs16 = readUShort();
 		if (ofs16 != readUShort())
 			ofs = ofs16 << 1;
 		else
