@@ -1,11 +1,11 @@
 /*
  * X Font Server for *.ttf Files
  *
- * $Id: xfstt.cc,v 1.18 2003/08/07 06:31:47 guillem Exp $
+ * $Id: xfstt.cc 127 2005-11-03 01:53:01Z guillem $
  *
  * Copyright (C) 1997-1999 Herbert Duerr
  * portions are (C) 1999 Stephen Carpenter and others
- * portions are (C) 2002-2003 Guillem Jover
+ * portions are (C) 2002-2005 Guillem Jover
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,8 +18,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Softaware
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -48,6 +48,7 @@
 #include "ttfn.h"
 #include "xfstt.h"
 #include "encoding.h"
+#include "mesg.h"
 
 #include <sys/types.h>
 #include <dirent.h>
@@ -65,10 +66,12 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
+#include <netdb.h>
 #include <time.h>
 #include <X11/fonts/FS.h>
 #include <X11/fonts/FSproto.h>
 #include <pwd.h>
+#include <errno.h>
 
 
 // if you want to read good code skip this hacked up file!
@@ -87,7 +90,7 @@ XFSFont xfsFont[MAXOPENFONTS];
 int MAGNIFY = 0;
 #endif /* MAGNIFY */
 
-U16 maxLastChar = 255;
+u16_t maxLastChar = 255;
 
 static unsigned infoSize, nameSize, aliasSize;
 static char *infoBase, *nameBase, *aliasBase;
@@ -97,7 +100,16 @@ char *pidfilename = PIDFILE;
 
 int defaultres = 0;
 const int default_port = 7101;
-int noTCP = 0;
+
+struct fs_conn {
+	bool listen_unix;
+	bool listen_inet;
+	int port;
+	int sd_max;
+	int *sd_list;
+	int sd_list_size;
+	int sd_list_used;
+};
 
 uid_t newuid = (uid_t)(-2);
 gid_t newgid = (uid_t)(-2);
@@ -156,7 +168,7 @@ ttSyncDir(FILE *infoFile, FILE *nameFile, char *ttdir, int gslist)
 	int ttdir_len = strlen(ttdir);
 
 	if (!gslist)
-		printf(_("xfstt: sync in directory \"%s/%s\"\n"), fontdir, ttdir);
+		info(_("Sync in directory \"%s/%s\".\n"), fontdir, ttdir);
 
 	DIR *dirp = opendir(".");
 
@@ -247,12 +259,11 @@ cachefile(const char *leafname)
 	struct stat statbuf;
 
 	if (stat(cachedir, &statbuf)) {
-		fprintf(stderr, _("xfstt: \"%s\" does not exist!\n"), cachedir);
+		error(_("directory \"%s\" does not exist!\n"), cachedir);
 		return 0;
 	}
 	if (!S_ISDIR(statbuf.st_mode)) {
-		fprintf(stderr, _("xfstt: \"%s\" is not a directory!\n"),
-			cachedir);
+		error(_("\"%s\" is not a directory!\n"), cachedir);
 		return 0;
 	}
 
@@ -260,7 +271,7 @@ cachefile(const char *leafname)
 	long maxlen = pathconf(cachedir, _PC_PATH_MAX);
 
 	if (maxlen != -1 && len > maxlen) {
-		fputs(_("xfstt: Cache directory name is too long\n"), stderr);
+		error(_("cache directory name is too long.\n"));
 		return 0;
 	}
 
@@ -277,7 +288,7 @@ ttSyncAll(int gslist = 0)
 		debug("TrueType syncing\n");
 
 	if (chdir(fontdir)) {
-		fprintf(stderr, _("xfstt: \"%s\" does not exist!\n"), fontdir);
+		error(_("directory \"%s\" does not exist!\n"), fontdir);
 		return -1;
 	}
 	DIR *dirp = opendir(".");
@@ -297,7 +308,7 @@ ttSyncAll(int gslist = 0)
 	delete ttnamefilename;
 
 	if (infoFile <= 0 || nameFile <= 0) {
-		fputs(_("xfstt: Cannot write to font database!\n"), stderr);
+		error(_("cannot write to font database!\n"));
 		return -1;
 	}
 
@@ -322,10 +333,10 @@ ttSyncAll(int gslist = 0)
 
 	if (nfonts > 0) {
 		if (!gslist)
-			printf(_("Found %d fonts.\n"), nfonts);
+			info(_("Found %d fonts.\n"), nfonts);
 	} else {
-		printf(_("No valid truetype fonts found!\n"));
-		printf(_("Please put some *.ttf fonts into \"%s\"\n"), fontdir);
+		error(_("no valid truetype fonts found!\n"));
+		info(_("Please put some *.ttf fonts into \"%s\".\n"), fontdir);
 	}
 
 	return nfonts;
@@ -426,7 +437,7 @@ listTTFNFonts(char *pattern, int index, char *buf)
 		char *name = alias;
 		while (--len > 0 && *(alias++) != '\"');
 		*buf = alias - name - 1;
-		strncpy(buf + 1, name, *(U8 *)buf);
+		strncpy(buf + 1, name, *(u8_t *)buf);
 		return *buf + 1;
 	}
 #endif
@@ -545,10 +556,9 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 		fp->pixel[0], fp->pixel[2], fp->pixel[3], fp->pixel[1]);
 
 	xfs->fe.buflen = MAXFONTBUFSIZE;
-	while (!(xfs->fe.buffer = (U8 *)allocMem(xfs->fe.buflen)))
+	while (!(xfs->fe.buffer = (u8_t *)allocMem(xfs->fe.buflen)))
 		if ((xfs->fe.buflen >>= 1) < MINFONTBUFSIZE) {
-			fputs(_("xfstt: entering memory starved mode\n"),
-			      stderr);
+			error(_("entering memory starved mode.\n"));
 			xfs->fid = 0;
 			return 0;
 		}
@@ -557,7 +567,7 @@ openFont(TTFont *ttFont, FontParams *fp, Rasterizer *raster,
 
 	int used = (xfs->fe.bitmaps + xfs->fe.bmplen) - xfs->fe.buffer;
 	int bmpoff = xfs->fe.bitmaps - xfs->fe.buffer;
-	xfs->fe.buffer = (U8 *)shrinkMem(xfs->fe.buffer, used);
+	xfs->fe.buffer = (u8_t *)shrinkMem(xfs->fe.buffer, used);
 	if (xfs->fe.buffer) {
 		xfs->fe.buflen = used;
 		xfs->fe.bitmaps = xfs->fe.buffer + bmpoff;
@@ -621,25 +631,39 @@ openTTFN(Rasterizer *raster, char *ttfnName, FontParams *fp, int fid)
 	// set attribute defaults
 
 	switch (m_index) {
-	case 0:	// use fp defaults		//fall through
-	case 1:	fp->point[1] = fp->point[0];	//fall through
-	case 2:	fp->point[2] = 0;		//fall through
-	case 3:	fp->point[3] = -fp->point[2];	//fall through
-	default: break;
+	case 0:
+		// use fp defaults		//fall through
+	case 1:
+		fp->point[1] = fp->point[0];	//fall through
+	case 2:
+		fp->point[2] = 0;		//fall through
+	case 3:
+		fp->point[3] = -fp->point[2];	//fall through
+	default:
+		break;
 	}
 
 	switch (p_index) {
-	case 0:	// use fp defaults		//fall through
-	case 1:	fp->pixel[1] = fp->pixel[0];	//fall through
-	case 2:	fp->pixel[2] = 0;		//fall through
-	case 3:	fp->pixel[3] = -fp->pixel[2];	//fall through
-	default: break;
+	case 0:
+		// use fp defaults		//fall through
+	case 1:
+		fp->pixel[1] = fp->pixel[0];	//fall through
+	case 2:
+		fp->pixel[2] = 0;		//fall through
+	case 3:
+		fp->pixel[3] = -fp->pixel[2];	//fall through
+	default:
+		break;
 	}
 
 	switch (r_index) {
-	case 0:	break; // use fp defaults
-	case 1:	fp->resolution[1] = fp->resolution[0]; break;
-	default: break;
+	case 0:
+		break; // use fp defaults
+	case 1:
+		fp->resolution[1] = fp->resolution[0];
+		break;
+	default:
+		break;
 	}
 
 	TTFNdata *ttfn = (TTFNdata *)(infoBase + sizeof(TTFNheader));
@@ -677,14 +701,14 @@ openXLFD(Rasterizer *raster, char *xlfdName, FontParams *fp, int fid)
 		*p = tolower(*p);
 		if (*p == '-')
 			switch (++delim) {
-			case  7:	// pixelsize
+			case 7:		// pixelsize
 				fp->pixel[0] = fp->pixel[1] = xatoi(++p);
 				*p = 0;
 				break;
-			case  8:	// pointsize
+			case 8:		// pointsize
 				fp->point[0] = fp->point[1] = xatoi(++p)/10;
 				break;
-			case  9:	// x-resolution
+			case 9:		// x-resolution
 				fp->resolution[0] = xatoi(++p);
 				break;
 			case 10:	// y-resolution
@@ -707,7 +731,7 @@ openXLFD(Rasterizer *raster, char *xlfdName, FontParams *fp, int fid)
 
 	TTFNdata* ttfn = (TTFNdata *)(infoBase + sizeof(TTFNheader));
 	// XXX: linear search should be replaced
-	for (; (char *) ttfn < infoBase + infoSize; ++ttfn) {
+	for (; (char *)ttfn < infoBase + infoSize; ++ttfn) {
 		char *file = nameBase + ttfn->nameOfs + ttfn->nameLen + 1;
 		char *xlfd = file + ttfn->pathLen + 1;
 		char *p = xlfdName;
@@ -733,7 +757,7 @@ openTTFdb()
 	infoSize = nameSize = aliasSize = 0;
 
 	if (chdir(fontdir)) {
-		fprintf(stderr, _("xfstt: \"%s\" does not exist!\n"), fontdir);
+		error(_("directory \"%s\" does not exist!\n"), fontdir);
 		return 0;
 	}
 
@@ -743,7 +767,7 @@ openTTFdb()
 
 	int fd = open(ttinfofilename, O_RDONLY);
 	if (0 >= fd) {
-		fputs(_("Can not open font database!\n"), stderr);
+		error(_("cannot open font database!\n"));
 		delete ttinfofilename;
 		return 0;
 	}
@@ -757,12 +781,12 @@ openTTFdb()
 
 	if (infoSize <= sizeof(TTFNheader)
 	    || strncmp(infoBase, "TTFNINFO", 8)) {
-		fputs(_("Corrupt font database!\n"), stderr);
+		error(_("corrupt font database!\n"));
 		return 0;
 	}
 
 	if (((TTFNheader *)infoBase)->version != TTFN_VERSION) {
-		fputs(_("Wrong font database version!\n"), stderr);
+		error(_("wrong font database version!\n"));
 		return 0;
 	}
 
@@ -772,7 +796,7 @@ openTTFdb()
 
 	fd = open(ttnamefilename, O_RDONLY);
 	if (0 >= fd) {
-		fputs(_("Can not open font database!\n"), stderr);
+		error(_("cannot open font database!\n"));
 		delete ttnamefilename;
 		return 0;
 	}
@@ -785,12 +809,12 @@ openTTFdb()
 
 	if (nameSize <= sizeof(TTFNheader)
 	    || strncmp(nameBase, "TTFNNAME", 8)) {
-		fputs(_("Corrupt font database!\n"), stderr);
+		error(_("corrupt font database!\n"));
 		return 0;
 	}
 
 	if (((TTFNheader *)nameBase)->version != TTFN_VERSION) {
-		fputs(_("Wrong font database version!\n"), stderr);
+		error(_("wrong font database version!\n"));
 		return 0;
 	}
 
@@ -820,72 +844,173 @@ closeTTFdb()
 	infoSize = nameSize = aliasSize = 0;
 }
 
-static int
-prepare2connect(int portno)
+#if defined(HAVE_IPV6)
+static bool
+fs_connection_setup_inet(fs_conn &conn, struct addrinfo *res)
 {
-	static struct sockaddr_un s_unix;
-	static struct sockaddr_in s_inet;
-	static int sd_unix = 0;
-	static int sd_inet = 0;
+	int inet_ports = 0;
+	const int on = 1;
 
-	if (!sd_unix) {
+	for (; res && conn.sd_list_used < conn.sd_list_size; res = res->ai_next) {
+		int sd;
+
+		sd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+		if (sd < 0)
+			continue;
+
+#if defined(HAVE_IPV6_V6ONLY)
+		if (res->ai_family == PF_INET6 &&
+		    setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0) {
+			error(_("setting socket option (IPv6 only).\n"));
+			close(sd);
+			continue;
+		}
+#endif
+
+		if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
+			error(_("setting socket option (reuseaddr).\n"));
+			close(sd);
+			continue;
+		}
+
+		if (bind(sd, res->ai_addr, res->ai_addrlen) < 0) {
+			close(sd);
+			continue;
+		}
+
+		listen(sd, 1);
+		conn.sd_list[conn.sd_list_used++] = sd;
+		inet_ports++;
+	}
+
+	if (!inet_ports) {
+		error(_("cannot open TCP/IP port %d, try another port; %s!\n"),
+		      conn.port, strerror(errno));
+		return false;
+	} else
+		return true;
+}
+#endif
+
+static int
+fs_connection_setup(fs_conn &conn)
+{
+	int sd;
+
+	if (conn.listen_unix) {
+		struct sockaddr_un s_unix;
 		mode_t old_umask;
 
 		// prepare unix connection
-		sd_unix = socket(PF_UNIX, SOCK_STREAM, 0);
+		sd = socket(PF_UNIX, SOCK_STREAM, 0);
 
 		s_unix.sun_family = AF_UNIX;
-		sprintf(s_unix.sun_path, "%s/fs%d", sockdir, portno);
+		sprintf(s_unix.sun_path, "%s/fs%d", sockdir, conn.port);
 		sockname = s_unix.sun_path;
 		mkdir(sockdir, 01777);
 		unlink(s_unix.sun_path);
 		old_umask = umask(0);
-		if (bind(sd_unix, (struct sockaddr *)&s_unix, sizeof(s_unix))) {
-			fprintf(stderr, _("Couldn't write to %s/\n"), sockdir);
-			fputs(_("Please check permissions.\n"), stderr);
+
+		if (bind(sd, (struct sockaddr *)&s_unix, sizeof(s_unix)) < 0) {
+			error(_("could not write to %s/, please check "
+			      "permissions.\n"), sockdir);
+			close(sd);
+		} else {
+			listen(sd, 1);
+			conn.sd_list[conn.sd_list_used++] = sd;
 		}
 		umask(old_umask);
-		listen(sd_unix, 1);	// only one connection
 	}
 
-	if (!noTCP && !sd_inet) {
-		// prepare inet connection
-		sd_inet = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+	if (conn.listen_inet) {
+#if defined(HAVE_IPV6)
+		struct addrinfo hints, *res;
+		char *service;
+		int err;
+
+		// prepare inet connections
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = PF_UNSPEC;
+		hints.ai_flags = AI_PASSIVE;
+		hints.ai_socktype = SOCK_STREAM;
+		asprintf(&service, "%d", conn.port);
+
+		err = getaddrinfo(NULL, service, &hints, &res);
+
+		free(service);
+
+		if (err) {
+			perror(gai_strerror(err));
+		} else {
+			fs_connection_setup_inet(conn, res);
+		}
+
+		freeaddrinfo(res);
+#else
+		struct sockaddr_in s_inet;
+
+		// prepare inet connections
+		sd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
 
 		s_inet.sin_family = AF_INET;
-		s_inet.sin_port = htons(portno);
+		s_inet.sin_port = htons(conn.port);
 		s_inet.sin_addr.s_addr = htonl(INADDR_ANY);
-		if (bind(sd_inet, (struct sockaddr *)&s_inet, sizeof(s_inet))) {
-			fprintf(stderr, _("Cannot open TCPIP port %d\n"), portno);
-			fprintf(stderr, _("Better try another port!\n"));
+
+		if (bind(sd, (struct sockaddr *)&s_inet, sizeof(s_inet)) < 0) {
+			error(_("cannot open TCP/IP port %d, "
+				"try another port; %s!\n"), conn.port,
+				strerror(errno));
+			close(sd);
 		}
-		listen(sd_inet, 1);	// only one connection
+		listen(sd, 1);
+		conn.sd_list[conn.sd_list_used++] = sd;
+#endif
 	}
 
-	fd_set sdlist;
-	FD_ZERO(&sdlist);
-	FD_SET(sd_unix, &sdlist);
-	if (!noTCP)
-		FD_SET(sd_inet, &sdlist);
-	int maxsd = (sd_inet > sd_unix) ? sd_inet : sd_unix;
-	select(maxsd + 1, &sdlist, 0L, 0L, 0L);
+	conn.sd_max = 0;
+
+	for (int n = 0; n < conn.sd_list_used; n++) {
+		if (conn.sd_list[n] > conn.sd_max)
+			conn.sd_max = conn.sd_list[n];
+	}
+
+	debug("connection setup (sockets = %d)\n", conn.sd_list_used);
+
+	return 0;
+}
+
+static int
+fs_connection_new(fs_conn &conn)
+{
+	int n;
+	fd_set sd_set;
+
+	FD_ZERO(&sd_set);
+
+	for (n = 0; n < conn.sd_list_used; n++)
+		FD_SET(conn.sd_list[n], &sd_set);
+
+	select(conn.sd_max + 1, &sd_set, 0L, 0L, 0L);
 
 	int sd = 0;
-	unsigned int saLength = sizeof(struct sockaddr);
-	if (FD_ISSET(sd_unix, &sdlist))
-		sd = accept(sd_unix, (struct sockaddr *)&s_unix, &saLength);
-	else if (!noTCP && FD_ISSET(sd_inet, &sdlist))
-		sd = accept(sd_inet, (struct sockaddr *)&s_inet, &saLength);
-	debug("accept(saLength = %d) = %d\n", saLength, sd);
+
+	for (n = 0; n < conn.sd_list_used; n++) {
+		if (FD_ISSET(conn.sd_list[n], &sd_set)) {
+			sd = accept(conn.sd_list[n], NULL, NULL);
+			break;
+		}
+	}
+
+	debug("accept(%d) = %d\n", conn.sd_list[n], sd);
 
 	return sd;
 }
 
 #define MAXREQSIZE 4096
-U8 buf[MAXREQSIZE + 256];
+u8_t buf[MAXREQSIZE + 256];
 
 static int
-connecting(int sd)
+fs_connecting(int sd)
 {
 	debug("Connecting\n");
 
@@ -897,7 +1022,7 @@ connecting(int sd)
 		return 0;
 
 	if (req->byteOrder != 'l' && req->byteOrder != 'B') {
-		fputs(_("xfstt: invalid byteorder, giving up\n"), stderr);
+		error(_("invalid byteorder, giving up.\n"));
 		return 0;
 	}
 
@@ -905,9 +1030,10 @@ connecting(int sd)
 	      (req->byteOrder == 'l') ? "little" : "big");
 	debug("version %d.%d\n", req->major_version, req->minor_version);
 
-	if ((req->byteOrder == 'l' && (*(U32 *)req & 0xff) != 'l')
-	    || (req->byteOrder == 'B' && ((*(U32 *)req >> 24) & 0xff) != 'B')) {
-		fputs(_("xfstt: byteorder mismatch, giving up\n"), stderr);
+	if ((req->byteOrder == 'l' && (*(u32_t *)req & 0xff) != 'l')
+	    || (req->byteOrder == 'B' && ((*(u32_t *)req >> 24) & 0xff) != 'B'))
+	{
+		error(_("byteorder mismatch, giving up.\n"));
 		return 0;
 	}
 
@@ -940,20 +1066,18 @@ connecting(int sd)
 }
 
 static void
-fixup_bitmap(FontExtent *fe, U32 hint)
+fixup_bitmap(FontExtent *fe, u32_t hint)
 {
 	int format = ((hint >> 8) & 3) + 3;
 	if (format < LOGSLP) {
-		fputs(_("xfstt: scanline length error!\n"), stderr);
-		fprintf(stderr,
-			_("recompile xfstt with LOGSLP defined as %d!\n"),
-			format <= 3 ? 3 : format);
+		error(_("scanline length error! recompile xfstt with LOGSLP "
+		      "defined as %d!\n"), format <= 3 ? 3 : format);
 	}
 
 	if ((hint ^ fe->bmpFormat) == 0)
 		return;
 
-	register U8 *p, *end = fe->bitmaps + fe->bmplen;
+	register u8_t *p, *end = fe->bitmaps + fe->bmplen;
 	if ((fe->bmpFormat ^ hint) & BitmapFormatMaskByte) {
 		debug("slpswap SLP=%d\n", LOGSLP);
 		p = fe->bitmaps;
@@ -962,17 +1086,17 @@ fixup_bitmap(FontExtent *fe, U32 hint)
 			break;
 		case 4:
 			for (; p < end; p += 2)
-				*(U16 *)p = bswaps(*(U16 *)p);
+				*(u16_t *)p = bswaps(*(u16_t *)p);
 			break;
 		case 5:
 			for (; p < end; p += 4)
-				*(U32 *)p = bswapl(*(U32 *)p);
+				*(u32_t *)p = bswapl(*(u32_t *)p);
 			break;
 		case 6:
 			for (; p < end; p += 8) {
-				U32 tmp = *(U32 *)p;
-				*(U32 *)(p + 0) = bswapl(*(U32 *)(p + 4));
-				*(U32 *)(p + 4) = bswapl(tmp);
+				u32_t tmp = *(u32_t *)p;
+				*(u32_t *)(p + 0) = bswapl(*(u32_t *)(p + 4));
+				*(u32_t *)(p + 4) = bswapl(tmp);
 			}
 			break;
 		}
@@ -980,7 +1104,7 @@ fixup_bitmap(FontExtent *fe, U32 hint)
 
 	if ((fe->bmpFormat ^ hint) & BitmapFormatMaskBit) {
 		debug("bitswap\n");
-		U8 map[16] = {0, 8, 4, 12, 2, 10, 6, 14,
+		u8_t map[16] = {0, 8, 4, 12, 2, 10, 6, 14,
 			      1, 9, 5, 13, 3, 11, 7, 15};
 		for (p = fe->bitmaps; p < end; ++p)
 			*p = (map[*p & 15] << 4) | map[(*p >> 4) & 15];
@@ -991,36 +1115,36 @@ fixup_bitmap(FontExtent *fe, U32 hint)
 		p = fe->bitmaps;
 		if (LOGSLP == 3 && format == 4) {
 			for (; p < end; p += 2)
-				*(U16 *)p = bswaps(*(U16 *)p);
+				*(u16_t *)p = bswaps(*(u16_t *)p);
 		} else if (LOGSLP == 3 && format == 5) {
 			for (; p < end; p += 4)
-				*(U32 *)p = bswapl(*(U32 *)p);
+				*(u32_t *)p = bswapl(*(u32_t *)p);
 		} else if (LOGSLP == 3 && format == 6) {
 			for (; p < end; p += 8) {
-				U32 tmp = *(U32 *)p;
-				*(U32 *)(p + 0) = bswapl(*(U32 *)(p + 4));
-				*(U32 *)(p + 4) = bswapl(tmp);
+				u32_t tmp = *(u32_t *)p;
+				*(u32_t *)(p + 0) = bswapl(*(u32_t *)(p + 4));
+				*(u32_t *)(p + 4) = bswapl(tmp);
 			}
 		} else if (LOGSLP == 4 && format == 5) {
 			for (; p < end; p += 4) {
-				U16 tmp = *(U16 *)p;
-				*(U16 *)(p + 0) = *(U16 *)(p + 2);
-				*(U16 *)(p + 2) = tmp;
+				u16_t tmp = *(u16_t *)p;
+				*(u16_t *)(p + 0) = *(u16_t *)(p + 2);
+				*(u16_t *)(p + 2) = tmp;
 			}
 		} else if (LOGSLP == 5 && format == 6) {
 			for (; p < end; p += 8) {
-				U32 tmp = *(U32 *)p;
-				*(U32 *)(p + 0) = *(U32 *)(p + 4);
-				*(U32 *)(p + 4) = tmp;
+				u32_t tmp = *(u32_t *)p;
+				*(u32_t *)(p + 0) = *(u32_t *)(p + 4);
+				*(u32_t *)(p + 4) = tmp;
 			}
 		} else { // (LOGSLP == 4 && format == 6)
 			for (; p < end; p += 8) {
-				U16 tmp = *(U16 *)p;
-				*(U16 *)(p + 0) = *(U16 *)(p + 6);
-				*(U16 *)(p + 6) = tmp;
-				tmp = *(U16 *)(p + 2);
-				*(U16 *)(p + 2) = *(U16 *)(p + 4);
-				*(U16 *)(p + 4) = tmp;
+				u16_t tmp = *(u16_t *)p;
+				*(u16_t *)(p + 0) = *(u16_t *)(p + 6);
+				*(u16_t *)(p + 6) = tmp;
+				tmp = *(u16_t *)(p + 2);
+				*(u16_t *)(p + 2) = *(u16_t *)(p + 4);
+				*(u16_t *)(p + 4) = tmp;
 			}
 		}
 	}
@@ -1029,7 +1153,7 @@ fixup_bitmap(FontExtent *fe, U32 hint)
 }
 
 static int
-send_fserror(int sd, int seqno)
+fs_error_length(int sd, int seqno)
 {
 	fsError reply;
 
@@ -1042,12 +1166,12 @@ send_fserror(int sd, int seqno)
 }
 
 static int
-check_length(int sd, int seqno, fsReq *req, int expected_size)
+fs_check_length(int sd, int seqno, fsReq *req, int expected_size)
 {
 	if (req->length < (expected_size >> 2)) {
 		debug("packet size mismatch: %d received bytes, "
 		      "%d expected bytes\n", req->length << 2, expected_size);
-		send_fserror(sd, seqno);
+		fs_error_length(sd, seqno);
 		return 0;
 	} else {
 		return 1;
@@ -1055,7 +1179,7 @@ check_length(int sd, int seqno, fsReq *req, int expected_size)
 }
 
 static int
-working(int sd, Rasterizer *raster, char *replybuf)
+fs_working(int sd, Rasterizer *raster, char *replybuf)
 {
 	FontParams fp0 = {{0, 0, 0, 0}, {0, 0, 0, 0}, {VGARES, VGARES}, 0}, fp;
 	int event_mask = 0;
@@ -1070,10 +1194,10 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			return l;
 
 #ifdef DEBUG
-		printf("===STARTREQ=========== %d\n", l);
+		debug("===STARTREQ=========== %d\n", l);
 		for (i = 0; i < sz_fsReq; ++i)
-			printf("%02X ", buf[i]);
-		printf("\n");
+			debug("%02X ", buf[i]);
+		debug("\n");
 		sync();
 #endif
 
@@ -1082,7 +1206,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 		if (length > MAXREQSIZE) {
 			debug("too much data: %d bytes (max=%d)\n",
 			      length, MAXREQSIZE);
-			send_fserror(sd, seqno);
+			fs_error_length(sd, seqno);
 			break;
 		}
 
@@ -1094,13 +1218,13 @@ working(int sd, Rasterizer *raster, char *replybuf)
 
 #ifdef DEBUG
 		for (i = sz_fsReq; i < length; ++i) {
-			printf("%02X ", buf[i]);
+			debug("%02X ", buf[i]);
 			if ((i & 3) == 3)
-				printf(" ");
+				debug(" ");
 			if ((i & 15) == (15 - sz_fsReq))
-				printf("\n");
+				debug("\n");
 		}
-		printf("\n===ENDREQ============= %d\n", length);
+		debug("\n===ENDREQ============= %d\n", length);
 		sync();
 #endif
 
@@ -1236,7 +1360,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			int expected_size = numres * sz_fsResolution
 					    + sz_fsSetResolutionReq;
 
-			if (!check_length(sd, seqno, fsreq, expected_size))
+			if (!fs_check_length(sd, seqno, fsreq, expected_size))
 				break;
 
 			fsResolution *res = (fsResolution *)(req + 1);
@@ -1285,7 +1409,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			char *pattern = (char *)(req + 1);
 			int expected_size = sz_fsListFontsReq + req->nbytes;
 
-			if (!check_length(sd, seqno, fsreq, expected_size))
+			if (!fs_check_length(sd, seqno, fsreq, expected_size))
 				break;
 
 			pattern[req->nbytes] = 0;
@@ -1369,7 +1493,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			{
 			fsOpenBitmapFontReq *req = (fsOpenBitmapFontReq *)buf;
 			char *fontName = (char *)(req + 1) + 1;
-			fontName[*(U8 *)(req + 1)] = 0;
+			fontName[*(u8_t *)(req + 1)] = 0;
 			debug("FS_OpenBitmapFont \"%s\"", fontName);
 
 			raster->format = (req->format_hint >> 8) & 3;
@@ -1413,7 +1537,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 				fsQueryXInfoReply s1;
 				fsPropInfo s2;
 				fsPropOffset s3;
-				U32 dummyName, dummyValue;
+				u32_t dummyName, dummyValue;
 			} reply;
 
 			reply.s1.type = FS_Reply;
@@ -1430,14 +1554,14 @@ working(int sd, Rasterizer *raster, char *replybuf)
 
 			reply.s1.font_hdr_char_range_min_char_high
 				= reply.s1.font_header_default_char_high
-				= (U8)(fi->firstChar >> 8);
+				= (u8_t)(fi->firstChar >> 8);
 			reply.s1.font_hdr_char_range_min_char_low
 				= reply.s1.font_header_default_char_low
-				= (U8)fi->firstChar;
+				= (u8_t)fi->firstChar;
 			reply.s1.font_hdr_char_range_max_char_high
-				= (U8)(fi->lastChar >> 8);
+				= (u8_t)(fi->lastChar >> 8);
 			reply.s1.font_hdr_char_range_max_char_low
-				= (U8)fi->lastChar;
+				= (u8_t)fi->lastChar;
 
 			debug("minchar = 0x%02X%02X, ",
 			      reply.s1.font_hdr_char_range_min_char_high,
@@ -1505,15 +1629,15 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			int expected_size = sz_fsQueryXExtents8Req
 				            + req->num_ranges * item_size;
 
-			if (!check_length(sd, seqno, fsreq, expected_size))
+			if (!fs_check_length(sd, seqno, fsreq, expected_size))
 				break;
 
 			if (req->reqType == FS_QueryXExtents8) {
 				/*
 				 * Convert to QueryXExtents16 request
 				 */
-				U8 *p8 = (U8 *)(req + 1);
-				U16 *p16 = (U16 *)p8;
+				u8_t *p8 = (u8_t *)(req + 1);
+				u16_t *p16 = (u16_t *)p8;
 				for (i = req->num_ranges; --i >= 0;)
 					p16[i] = htons(p8[i]);
 			}
@@ -1523,7 +1647,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 				break;
 
 			fsXCharInfo *ext0 = (fsXCharInfo *)replybuf, *ext = ext0;
-			U16 *ptr = (U16 *)(req + 1);
+			u16_t *ptr = (u16_t *)(req + 1);
 			int nranges = req->num_ranges;
 			if (req->range) {
 				ptr[nranges] = htons(xfs->fi.lastChar);
@@ -1536,7 +1660,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 					ptr[0] = ntohs(ptr[0]);
 					ptr[1] = ntohs(ptr[1]);
 					debug("rg %d..%d\n",ptr[0],ptr[1]);
-					for (U16 j = ptr[0]; j <= ptr[1]; ++j)
+					for (u16_t j = ptr[0]; j <= ptr[1]; ++j)
 						(ext++)->left = j;
 				}
 			} else
@@ -1548,7 +1672,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			reply.sequenceNumber = seqno;
 			reply.num_extents = ext - ext0;
 			reply.length = (sizeof(reply) + 3
-				       + ((U8 *)ext - (U8 *)ext0)) >> 2;
+				       + ((u8_t *)ext - (u8_t *)ext0)) >> 2;
 
 			CharInfo *ci = (CharInfo *)xfs->fe.buffer;
 
@@ -1582,7 +1706,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 #endif
 			}
 			write(sd, (void *)&reply, sizeof(reply));
-			write(sd, (void *)ext0, (U8 *)ext - (U8 *)ext0);
+			write(sd, (void *)ext0, (u8_t *)ext - (u8_t *)ext0);
 			}
 			break;
 
@@ -1601,15 +1725,15 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			int expected_size = sz_fsQueryXBitmaps8Req
 					    + req->num_ranges * item_size;
 
-			if (!check_length(sd, seqno, fsreq, expected_size))
+			if (!fs_check_length(sd, seqno, fsreq, expected_size))
 				break;
 
 			if (req->reqType == FS_QueryXBitmaps8) {
 				/*
 				 * Convert to QueryXBitmaps16 request
 				 */
-				U8 *p8 = (U8 *)(req + 1);
-				U16 *p16 = (U16 *)p8;
+				u8_t *p8 = (u8_t *)(req + 1);
+				u16_t *p16 = (u16_t *)p8;
 				for (i = req->num_ranges; --i >= 0;)
 					p16[i] = ntohs(p8[i]);
 			}
@@ -1621,7 +1745,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			fixup_bitmap(&xfs->fe, req->format);
 
 			fsOffset32 *ofs0 = (fsOffset32 *)replybuf, *ofs = ofs0;
-			U16 *ptr = (U16 *)(req + 1);
+			u16_t *ptr = (u16_t *)(req + 1);
 			int nranges = req->num_ranges;
 			if (req->range) {
 				ptr[nranges] = htons(xfs->fi.lastChar);
@@ -1634,7 +1758,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 					ptr[0] = ntohs(ptr[0]);
 					ptr[1] = ntohs(ptr[1]);
 					debug("rg %d..%d\n",ptr[0],ptr[1]);
-					for (U16 j = ptr[0]; j <= ptr[1]; ++j)
+					for (u16_t j = ptr[0]; j <= ptr[1]; ++j)
 						(ofs++)->position = j;
 				}
 			} else
@@ -1665,7 +1789,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 				ofs->length = ci->length;
 				if (ci->tmpofs < 0) {
 					if (bmp + ci->length < replylimit) {
-						U8 *src = xfs->fe.bitmaps;
+						u8_t *src = xfs->fe.bitmaps;
 						src += ci->offset;
 						memcpy(bmp, src, ci->length);
 						ci->tmpofs = bmp - bmp0;
@@ -1685,9 +1809,9 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			reply.nbytes = bmp - bmp0;
 #if 1
 			reply.length = (sizeof(reply) + reply.nbytes + 3
-				       + ((U8 *)ofs - (U8 *)ofs0)) >> 2;
+				       + ((u8_t *)ofs - (u8_t *)ofs0)) >> 2;
 			write(sd, (void *)&reply, sizeof(reply));
-			write(sd, (void *)ofs0, (U8 *)ofs - (U8 *)ofs0);
+			write(sd, (void *)ofs0, (u8_t *)ofs - (u8_t *)ofs0);
 			write(sd, (void *)bmp0, (reply.nbytes + 3) & ~3);
 #else
 {
@@ -1695,9 +1819,9 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			reply.nbytes = 0;
 			reply.replies_hint = 1;
 			reply.length = (sizeof(reply)
-				       + ((U8 *)ofs - (U8 *)ofs0)) >> 2;
+				       + ((u8_t *)ofs - (u8_t *)ofs0)) >> 2;
 			write(sd, (void *)&reply, sizeof(reply));
-			write(sd, (void *)ofs0, (U8 *)ofs - (U8 *)ofs0);
+			write(sd, (void *)ofs0, (u8_t *)ofs - (u8_t *)ofs0);
 
 			reply.nbytes = nbytes;
 			reply.replies_hint = 0;
@@ -1740,7 +1864,7 @@ working(int sd, Rasterizer *raster, char *replybuf)
 			}
 			break;
 		}
-	debug("done.\n");
+		debug("done.\n");
 	}
 
 	return 0;
@@ -1780,10 +1904,10 @@ main(int argc, char **argv)
 {
 	int multiConnection = 1;
 	int inetdConnection = 0;
-	int portno = default_port;
 	int gslist = 0;
 	int sync_db = 0;
 	int daemon = 0;
+	fs_conn fs_conn;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -1791,6 +1915,13 @@ main(int argc, char **argv)
 
 	Encoding::getDefault(encodings, MAXENC);
 
+	fs_conn.listen_unix = true;
+	fs_conn.listen_inet = true;
+	fs_conn.port = default_port;
+	fs_conn.sd_list_size = 16;
+	fs_conn.sd_list_used = 0;
+	fs_conn.sd_list = new int [fs_conn.sd_list_size];
+
 	for (int i = 1; i < argc; ++i) {
 		if (!strcmp(argv[i], "--gslist")) {
 			gslist = sync_db = 1;
@@ -1798,18 +1929,18 @@ main(int argc, char **argv)
 			sync_db = 1;
 		} else if (!strcmp(argv[i], "--port")) {
 			if (i <= argc)
-				portno = xatoi(argv[++i]);
-			if (!portno) {
-				fputs(_("Illegal port number!\n"), stderr);
-				portno = default_port;
+				fs_conn.port = xatoi(argv[++i]);
+			if (!fs_conn.port) {
+				error(_("illegal port number!\n"));
+				fs_conn.port = default_port;
 			}
 		} else if (!strcmp(argv[i], "--notcp")) {
-			noTCP = 1;
+			fs_conn.listen_inet = false;
 		} else if (!strcmp(argv[i], "--res")) {
 			if (i <= argc)
 				defaultres = xatoi(argv[++i]);
 			if (!defaultres)
-				fputs(_("Illegal default resolution!\n"), stderr);
+				error(_("illegal default resolution!\n"));
 		} else if (!strcmp(argv[i], "--dir")) {
 			fontdir = argv[++i];
 		} else if (!strcmp(argv[i], "--user")) {
@@ -1821,13 +1952,13 @@ main(int argc, char **argv)
 		} else if (!strcmp(argv[i], "--encoding")) {
 			char *maplist = argv[++i];
 			if (!Encoding::parse(maplist, encodings, MAXENC)) {
-				fprintf(stderr, _("Illegal encoding!\n"));
-				fprintf(stderr, _("Valid encodings are:\n"));
+				error(_("illegal encoding!\n"));
+				info(_("valid encodings are:\n"));
 				for (Encoding *maps = 0;;) {
 					maps = Encoding::enumerate(maps);
 					if (!maps)
 						exit(0);
-					fprintf(stderr, "\t%s\n", maps->strName);
+					info("\t%s\n", maps->strName);
 				}
 			}
 		} else if (!strcmp(argv[i], "--help")) {
@@ -1841,8 +1972,8 @@ main(int argc, char **argv)
 			multiConnection = 0;
 		} else if (!strcmp(argv[i], "--unstrap")) {
 			maxLastChar = UNSTRAPLIMIT;
-			printf(_("xfstt unstrapped: you must start X11 "
-				"with \"-deferglyphs 16\" option!\n"));
+			warning(_("[unstrapped] you must start X11 with "
+				"\"-deferglyphs 16\" option!\n"));
 		} else if (!strcmp(argv[i], "--daemon")) {
 			daemon = 1;
 		} else {
@@ -1853,15 +1984,14 @@ main(int argc, char **argv)
 
 	if (sync_db) {
 		if (ttSyncAll(gslist) <= 0)
-			fputs(_("xfstt: sync failed\n"), stderr);
+			error(_("sync failed.\n"));
 		cleanupMem();
 		return 0;
 	}
 
 	if (inetdConnection && multiConnection) {
 		multiConnection = 0;
-		fprintf(stderr,
-			_("xfstt: --inetd and --multi option collission\n"));
+		error(_("--inetd and --multi option collission.\n"));
 		// we don't know what to do ...so exit.
 		return 1;
 	}
@@ -1885,8 +2015,8 @@ main(int argc, char **argv)
 	// Make a pid file for easy starting and killing like
 	// a good little daemon
 	if (multiConnection) {
-		
 		FILE *pidfile = fopen(pidfilename, "w");
+
 		if (pidfile) {
 			pid_t pid = getpid();
 			fprintf(pidfile, "%d\n", pid);
@@ -1903,24 +2033,25 @@ main(int argc, char **argv)
 			break;
 		closeTTFdb();
 
-		fprintf(stderr, _("xfstt: error opening TTF database!\n"));
-		fprintf(stderr, _("xfstt: reading \"%s\" to build it.\n"),
-			fontdir);
+		error(_("opening TTF database failed, while reading \"%s\" "
+		      "to build it.\n"), fontdir);
 
 		if (ttSyncAll() > 0 && openTTFdb() > 0)
 			break;
-		fputs(_("Creating a font database failed\n"), stderr);
+		error(_("creating a font database failed.\n"));
 		unlink(pidfilename);
 	}
 
 	signal(SIGCHLD, SIG_IGN); // We don't need no stinkinig zombies -sjc
 
-	if (retry <= 0)
-		fputs(_("xfstt: Good bye.\n"), stderr);
-	else do {
-		int sd = inetdConnection ? 0 : prepare2connect(portno);
+	fs_connection_setup(fs_conn);
 
-		if (connecting(sd)) {
+	if (retry <= 0)
+		error(_("good bye.\n"));
+	else do {
+		int sd = inetdConnection ? 0 : fs_connection_new(fs_conn);
+
+		if (fs_connecting(sd)) {
 			if (!multiConnection || !fork()) {
 				setuid(newuid);
 				setgid(newgid);
@@ -1928,7 +2059,7 @@ main(int argc, char **argv)
 				Rasterizer *raster = new Rasterizer();
 				char *replybuf = (char *)allocMem(MAXREPLYSIZE);
 
-				working(sd, raster, replybuf);
+				fs_working(sd, raster, replybuf);
 				deallocMem(replybuf, MAXREPLYSIZE);
 				delete raster;
 
